IGNORED_MODULES = {
    'asyncio': 'WARNING',
    'websockets': 'WARNING',
    'websockets.client': 'WARNING',
    'websockets.server': 'WARNING',
    'uvicorn': 'WARNING',
    'urllib3': 'WARNING',
    'pydub': 'WARNING',
    'aiortc': 'WARNING',
    'grpc': 'WARNING',
    'aioice': 'WARNING',
    'matplotlib': 'WARNING',
    'PIL': 'WARNING',
    'h5py': 'WARNING',
    'httpx': 'ERROR',
    'multipart': 'WARNING',
    'asyncio.coroutines': 'WARNING',
    'numba': 'WARNING',
    'av.logging': 'WARNING',
    'pyee': 'WARNING',
    'http.server': 'WARNING',
    'pymongo': 'WARNING',
    'pymongo.serverSelection': 'WARNING',
    'pymongo.monitoring': 'WARNING',
    'httpcore': 'ERROR',
    'openai': 'ERROR',
    'neo4j': 'ERROR'
}
